//
//  MesonSKStoreManager.swift
//  MesonWrap
//
//  Created by Iti Agrawal on 05/02/21.
//  Copyright © 2021 InMobi. All rights reserved.
//

import Foundation
import StoreKit
import WebKit

@objc public class MesonSKStoreManager: NSObject {
    
    private var skStoreVC: SKStoreProductViewController = SKStoreProductViewController()
    private var pollingManager: PollingManager?
    private var skanParameters: [String: Any]?
    var adWebView: WKWebView?
    weak var delegate: MesonSkStoreManagerDelegate?
    
    private enum SkStoreState {
        case presented
        case dismissed
    }
    private var state: SkStoreState = .dismissed
    
    /**
     * Initializes a MesonSKStoreManager instance.
     */
    @objc public override init() {
        super.init()
        self.skStoreVC.delegate = self
        self.registerNotofication()
    }
    
    @objc func start() {
        let loadSKStoreHandler: (() -> Void) = { [weak self] in
            self?.callLoadProduct()
        }
        self.getProcessedSkanParameters(loadSKStoreHandler: loadSKStoreHandler)
    }
    
    /// Stop Meson's skstore manager.
    /// Publisher need to call it when ad is out of view.
    @objc public func stop() {
        self.pollingManager?.suspend()
        self.pollingManager = nil
    }
}

// Functions to read webview
extension MesonSKStoreManager {
    
    @objc private func startTimer() {
        if self.state == .presented {
            return
        }
        guard self.adWebView != nil else {
            return
        }
        let openSKStoreHandler: (() -> Void) = {[weak self] in
            self?.isOpenSkStoreCalled()
        }
        
        // Polling interval is in second
        self.pollingManager = PollingManager.init(pollingInterval: 0.2)
        self.pollingManager?.eventHandler = openSKStoreHandler
        self.pollingManager?.resume()
    }
    
    private func isOpenSkStoreCalled() {
        DispatchQueue.main.async {
            self.adWebView?.evaluateJavaScript("isOpenSkStoreCalled()", completionHandler: { [weak self] (result, _) in
                if let shouldOpenSkStore = result as? Bool, shouldOpenSkStore {
                    self?.adWebView?.evaluateJavaScript("resetSkStoreCalled()")
                    self?.pollingManager?.suspend()
                    self?.showSKStore()
                }
            })
        }
    }
    
    private func showSKStore() {
        if let topController = MesonSkanUtils.getTopMostVC() {
            if topController == self.skStoreVC { return }
            topController.present(self.skStoreVC, animated: true, completion: { [weak self] in
                self?.state = .presented
                self?.giveSuccessCallback()
            })
        }
    }
    
    private func giveSuccessCallback() {
        self.delegate?.didShow(skStore: self.skStoreVC)
    }
    
    private func giveDidDismissCallback() {
        self.delegate?.willDismiss(skStore: self.skStoreVC)
    }
    
    private func callLoadProduct() {
        guard let skanParams = self.skanParameters else { return }
        self.skStoreVC.loadProduct(withParameters: skanParams, completionBlock: { [weak self](result, error) in
            if !result {
                self?.delegate?.didFailToLoad(error: SkanError.skStoreError)
            }
        })
    }
    
    private func getProcessedSkanParameters(loadSKStoreHandler: @escaping (() -> Void) ) {
        if #available(iOS 14.0, *) {
            self.adWebView?.evaluateJavaScript("getSKAdNetworkParams()", completionHandler: { [weak self] (result, error) in
                guard let params = result as? [String: Any]  else {
                    print("Invalid Skan parameters")
                    return
                }
                guard let skanParams = MesonSkanUtils.process(skanParams: params) else {
                    print("Invalid data type of Skan Parameters")
                    return
                }
                self?.skanParameters = skanParams
                self?.adWebView?.evaluateJavaScript(MesonSkanUtils.mraidInterception, completionHandler: nil)
                self?.adWebView?.evaluateJavaScript("shouldInterceptURL(true)", completionHandler: { (_, error) in
                    if error == nil {
                        self?.startTimer()
                        loadSKStoreHandler()
                    } else {
                        print("MesonWrap: Can't Preocess SKAN Ads with error: \(String(describing: error?.localizedDescription))")
                    }
                })
            })
        } else {
            print("Only supported for iOS 14 onwards")
        }
    }
    
    func registerNotofication() {
        NotificationCenter.default.addObserver(self, selector: #selector(stopTimer), name: UIApplication.didEnterBackgroundNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(startTimer), name: UIApplication.willEnterForegroundNotification, object: nil)
    }
    
    @objc private func stopTimer() {
        self.pollingManager?.suspend()
    }
}

extension MesonSKStoreManager: SKStoreProductViewControllerDelegate {
    public func productViewControllerDidFinish(_ viewController: SKStoreProductViewController) {
        self.state = .dismissed
        print("skstore closed")
        viewController.dismiss(animated: true, completion: { [weak self] in
            self?.giveDidDismissCallback()
            self?.pollingManager?.resume()
            print("sk store dismissed")
        })
    }
}
