//
//  MesonMoPubSkanUtils.swift
//  MesonWrap
//
//  Created by Iti Agrawal on 04/02/21.
//  Copyright © 2021 InMobi. All rights reserved.
//

import Foundation
import WebKit
import UIKit
import StoreKit
enum SkanError: Error {
    case skStoreError
    case skanParamsError
}

/**
 * A listener for receiving notifications during the lifecycle of a MesonSkStoreManager Object.
 */
@objc public protocol MesonSkStoreManagerDelegate {
    @objc optional func adview() -> UIView
    
    func didFailToLoad(error: Error)
    func didFailToShow(error: Error)
    func didShow(skStore: UIViewController)
    func willDismiss(skStore: UIViewController)
}

struct MesonSkanUtils {
    
    // GAM writes the definition of Mraid after rendering the ad.
    // Hence, rewriting Mraid interception from SDK.
    static let mraidInterception = """
        // Case 3 : mraid.open
        mraid.open_ = mraid.open;
        mraid.open = function(url) {
            if (window.shouldInterceptURLs === null) {
                // In case SDK has not yet set the flag, wait 1 second
                window.setTimeout(handleMraidOpen, 1000, url);
            } else {
                handleMraidOpen(url);
            }
        }
    """
    
    private static let ItunesItemKey = "itunesitem"
    private static let NetworkKey = "network"
    private static let CampaignKey = "campaign"
    private static let TimestampKey = "timestamp"
    private static let NonceKey = "nonce"
    private static let SignatureKey = "signature"
    private static let SourceAppStoreID = "sourceapp"
    private static let VersionKey = "version"
    
    static func getWebView(view: UIView) -> WKWebView? {
        if view.isKind(of: WKWebView.self) {
            // swiftlint:disable:next force_cast
            return (view as! WKWebView)
        } else {
            for subView in view.subviews {
                if getWebView(view: subView) != nil { return getWebView(view: subView) }
            }
        }
        return nil
    }
    
    static func getWebView(viewController: UIViewController) -> WKWebView? {
        let view = viewController.view
        return MesonSkanUtils.getWebView(view: view!)
    }
    
    static func getTopMostVC() -> UIViewController? {
        if var topController = UIApplication.shared.keyWindow?.rootViewController {
            while let presentedViewController = topController.presentedViewController {
                topController = presentedViewController
            }
            return topController
        }
        return nil
    }
    
    @available(iOS 14.0, *)
    static func process(skanParams: [String: Any]) -> [String: Any]? {
        var skanParameters: [String: Any] = [:]
        
        if let idString = skanParams[MesonSkanUtils.ItunesItemKey] as? String, !idString.isEmpty,
           let network = skanParams[MesonSkanUtils.NetworkKey] as? String, !network.isEmpty,
           let campaignString = skanParams[MesonSkanUtils.CampaignKey] as? String, !campaignString.isEmpty,
           let timestampString = skanParams[MesonSkanUtils.TimestampKey] as? String, !timestampString.isEmpty,
           let nonceString = skanParams[MesonSkanUtils.NonceKey] as? String, !nonceString.isEmpty,
           let signature = skanParams[MesonSkanUtils.SignatureKey] as? String, !signature.isEmpty,
           let sourceappString = skanParams[MesonSkanUtils.SourceAppStoreID] as? String, !sourceappString.isEmpty,
           let version = skanParams[MesonSkanUtils.VersionKey] as? String, !version.isEmpty {
            
            // Check if Campaign Id string is Number else fail
            guard let id = MesonSkanUtils.convertStringToNumber(string: idString),
                  let campaign = MesonSkanUtils.convertStringToNumber(string: campaignString),
                  let timestamp = MesonSkanUtils.convertStringToNumber(string: timestampString),
                  let sourceapp = MesonSkanUtils.convertStringToNumber(string: sourceappString),
                  let nonce = UUID.init(uuidString: nonceString) else {
                print("MesonWrap: Incorrect SKAN params received")
               return nil
            }
            
            skanParameters[SKStoreProductParameterITunesItemIdentifier] = id
            skanParameters[SKStoreProductParameterAdNetworkIdentifier] = network
            skanParameters[SKStoreProductParameterAdNetworkCampaignIdentifier] = campaign
            skanParameters[SKStoreProductParameterAdNetworkNonce] = nonce
            skanParameters[SKStoreProductParameterAdNetworkTimestamp] = timestamp
            skanParameters[SKStoreProductParameterAdNetworkAttributionSignature] = signature
            skanParameters[SKStoreProductParameterAdNetworkSourceAppStoreIdentifier] = sourceapp
            skanParameters[SKStoreProductParameterAdNetworkVersion] = version
            return skanParameters
        }
        print("MesonWrap: Incorrect SKAN params received")
        return nil
    }
}

extension MesonSkanUtils {
    static func convertStringToNumber(string: String) -> NSNumber? {
        if let myInteger = Int64(string) {
            return NSNumber(value: myInteger)
        }
        return nil
    }
}
